# Generated by roxygen2: do not edit by hand

S3method(predict,big_plsr)
S3method(print,summary.big_plsr)
S3method(summary,big_plsr)
export(.finalize_pls_fit)
export(bigPLSR_stream_kstats)
export(kf_pls_state_fit)
export(kf_pls_state_new)
export(kf_pls_state_update)
export(plot_pls_biplot)
export(plot_pls_bootstrap_coefficients)
export(plot_pls_bootstrap_scores)
export(plot_pls_individuals)
export(plot_pls_variables)
export(plot_pls_vip)
export(pls_bootstrap)
export(pls_cross_validate)
export(pls_cv_select)
export(pls_fit)
export(pls_information_criteria)
export(pls_predict_response)
export(pls_predict_scores)
export(pls_select_components)
export(pls_threshold)
export(pls_vip)
export(summarise_pls_bootstrap)
import(bigmemory)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,dev.new)
importFrom(grDevices,hcl.colors)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,boxplot)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(stats,as.formula)
importFrom(stats,cov)
importFrom(stats,is.empty.model)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,quantile)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,modifyList)
importFrom(utils,read.csv)
useDynLib(bigPLSR, .registration = TRUE)
